package com.ekexiu.project.storage.dicom;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import org.dcm4che3.data.Attributes;
import org.dcm4che3.data.Tag;
import org.dcm4che3.data.UID;
import org.dcm4che3.data.VR;
import org.dcm4che3.net.ApplicationEntity;
import org.dcm4che3.net.Association;
import org.dcm4che3.net.Connection;
import org.dcm4che3.net.Device;
import org.dcm4che3.net.DimseRSPHandler;
import org.dcm4che3.net.IncompatibleConnectionException;
import org.dcm4che3.net.Priority;
import org.dcm4che3.net.pdu.AAssociateRQ;
import org.dcm4che3.net.pdu.ExtendedNegotiation;
import org.dcm4che3.net.pdu.PresentationContext;
import org.dcm4che3.data.ElementDictionary;

public class DicomMoveImage extends Device {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5892662719692590663L;

	protected static String[] IVR_LE_FIRST = { UID.ImplicitVRLittleEndian, UID.ExplicitVRLittleEndian,
			UID.ExplicitVRBigEndianRetired };

	private static enum InformationModel {
		PatientRoot(UID.PatientRootQueryRetrieveInformationModelMOVE, "STUDY"), StudyRoot(
				UID.StudyRootQueryRetrieveInformationModelMOVE,
				"STUDY"), PatientStudyOnly(UID.PatientStudyOnlyQueryRetrieveInformationModelMOVERetired,
						"STUDY"), CompositeInstanceRoot(UID.CompositeInstanceRootRetrieveMOVE,
								"IMAGE"), HangingProtocol(UID.HangingProtocolInformationModelMOVE,
										null), ColorPalette(UID.ColorPaletteQueryRetrieveInformationModelMOVE, null);
		final String cuid;
		final String level;

		InformationModel(String cuid, String level) {
			this.cuid = cuid;
			this.level = level;
		}
	}

	private static final int[] DEF_IN_FILTER = { Tag.SOPInstanceUID, Tag.StudyInstanceUID, Tag.SeriesInstanceUID };

	private final ApplicationEntity ae = new ApplicationEntity("AIADS");
	private final Connection conn = new Connection();
	private final Connection remote = new Connection();
	private final AAssociateRQ rq = new AAssociateRQ();
	private int priority;
	private String destination;

	private Attributes keys = new Attributes();
	private int[] inFilter = DEF_IN_FILTER;
	private Association as;

	public DicomMoveImage() throws IOException {
		super("AIADS");
		addConnection(conn);
		addApplicationEntity(ae);
		ae.addConnection(conn);
	}
	public void addKey(int tag, String... ss) {
		VR vr = ElementDictionary.vrOf(tag, keys.getPrivateCreator(tag));
		keys.setString(tag, vr, ss);
	}

	public final void setInputFilter(int[] inFilter) {
		this.inFilter = inFilter;
	}

	public void setRemote(String title, String hostname, int port) {
		rq.setCalledAET(title);
		remote.setHostname(hostname);
		remote.setPort(port);
		conn.setReceivePDULength(Connection.DEF_MAX_PDU_LENGTH);
		conn.setSendPDULength(Connection.DEF_MAX_PDU_LENGTH);
		conn.setMaxOpsInvoked(0);
		conn.setMaxOpsPerformed(0);

		conn.setPackPDV(true);
		conn.setConnectTimeout(0);
		conn.setRequestTimeout(0);
		conn.setAcceptTimeout(0);
		conn.setReleaseTimeout(0);
		conn.setResponseTimeout(0);

		conn.setIdleTimeout(0);
		conn.setSocketCloseDelay(Connection.DEF_SOCKETDELAY);
		conn.setSendBufferSize(0);
		conn.setReceiveBufferSize(0);
		conn.setTcpNoDelay(true);
	}

	public void addAttribute(int tag, String value) {
		VR vr = ElementDictionary.vrOf(tag, keys.getPrivateCreator(tag));
		if (value == null) {

			if (vr == VR.SQ)
				keys.newSequence(tag, 1).add(new Attributes(0));
			else
				keys.setNull(tag, vr);
		} else {
			keys.setString(tag, vr, value);
		}
	}

	public void move(String title, String hostname, int port, String imageId)
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {

		this.setRemote(title, hostname, port);
		this.rq.addPresentationContext(
				new PresentationContext(1, UID.StudyRootQueryRetrieveInformationModelMOVE, IVR_LE_FIRST));
		this.keys.setString(Tag.QueryRetrieveLevel, VR.CS, "IMAGE");
		this.addAttribute(Tag.SOPInstanceUID, imageId);

		this.destination = "AIADS";
		ExecutorService executorService = Executors.newSingleThreadExecutor();
		ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
		this.setExecutor(executorService);
		this.setScheduledExecutor(scheduledExecutorService);

		try {
			as = ae.connect(conn, remote, rq);
			DimseRSPHandler rspHandler = new DimseRSPHandler(as.nextMessageID()) {

				@Override
				public void onDimseRSP(Association as, Attributes cmd, Attributes data) {
					super.onDimseRSP(as, cmd, data);
				}
			};

			as.cmove(UID.StudyRootQueryRetrieveInformationModelMOVE, Priority.NORMAL, keys, null, destination, rspHandler);
		} finally {
			if (as != null && as.isReadyForDataTransfer()) {
				as.waitForOutstandingRSP();
				as.release();
			}
			executorService.shutdown();
			scheduledExecutorService.shutdown();
		}

	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args)
			throws IOException, GeneralSecurityException, InterruptedException, IncompatibleConnectionException {
		DicomMoveImage m = new DicomMoveImage();
		DicomStoreService.startup();
		try {
			m.move("STPACS", "133.0.14.150", 104, "1.2.840.113663.1500.1.446929507.3.1.20190308.101952.92");
		} finally {
			DicomStoreService.stop();
		}

	}
}
