package com.ekexiu.project.storage.dicom;

import java.io.File;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.atomic.AtomicBoolean;

import org.dcm4che3.data.Attributes;
import org.dcm4che3.data.Tag;
import org.dcm4che3.data.UID;
import org.dcm4che3.data.VR;
import org.dcm4che3.io.DicomOutputStream;
import org.dcm4che3.net.ApplicationEntity;
import org.dcm4che3.net.Association;
import org.dcm4che3.net.Connection;
import org.dcm4che3.net.Device;
import org.dcm4che3.net.PDVInputStream;
import org.dcm4che3.net.Status;
import org.dcm4che3.net.TransferCapability;
import org.dcm4che3.net.pdu.PresentationContext;
import org.dcm4che3.net.service.BasicCEchoSCP;
import org.dcm4che3.net.service.BasicCStoreSCP;
import org.dcm4che3.net.service.DicomServiceException;
import org.dcm4che3.net.service.DicomServiceRegistry;
import org.dcm4che3.util.SafeClose;

public class DicomStoreService
{
	private static AtomicBoolean running = new AtomicBoolean(false);
	private static final String PART_EXT = ".part";
	private static Connection closeConn;

	synchronized public static void startup() throws IOException, GeneralSecurityException {
		if (running.get()) {
			return;
		}

		DicomStoreService service = new DicomStoreService();
		service.run();
	}

	synchronized public static void stop() {
		if (!running.get()) {
			return;
		}
		closeConn.unbind();
		running.set(false);

	}

	private final Device device = new Device("AIADS");
	private final ApplicationEntity ae = new ApplicationEntity("AIADS");
	private final Connection conn = new Connection();
	private File storageDir = new File("/ekexiu/storage_asd/download");
	private int status;
	private int responseDelay;
	private final BasicCStoreSCP cstoreSCP = new BasicCStoreSCP("*") {

		@Override
		protected void store(Association as, PresentationContext pc, Attributes rq, PDVInputStream data, Attributes rsp)
				throws IOException {
			try {
				rsp.setInt(Tag.Status, VR.US, status);
				if (storageDir == null)
					return;

				String cuid = rq.getString(Tag.AffectedSOPClassUID);
				String iuid = rq.getString(Tag.AffectedSOPInstanceUID);
				String tsuid = pc.getTransferSyntax();
				File file = new File(storageDir, iuid + PART_EXT);
				try {
					storeTo(as, as.createFileMetaInformation(iuid, cuid, tsuid), data, file);
					renameTo(as, file, new File(storageDir, iuid));
				} catch (Exception e) {
					file.delete();
					throw new DicomServiceException(Status.ProcessingFailure, e);
				}
			} finally {
				if (responseDelay > 0)
					try {
						Thread.sleep(responseDelay);
					} catch (InterruptedException ignore) {}
			}
		}

	};

	public DicomStoreService() {
		device.setDimseRQHandler(createServiceRegistry());
		device.addConnection(conn);
		device.addApplicationEntity(ae);
		ae.setAssociationAcceptor(true);
		ae.addConnection(conn);
	}

	private DicomServiceRegistry createServiceRegistry() {
		DicomServiceRegistry serviceRegistry = new DicomServiceRegistry();
		serviceRegistry.addDicomService(new BasicCEchoSCP());
		serviceRegistry.addDicomService(cstoreSCP);
		return serviceRegistry;
	}

	private void storeTo(Association as, Attributes fmi, PDVInputStream data, File file) throws IOException {
		file.getParentFile().mkdirs();
		DicomOutputStream out = new DicomOutputStream(file);
		try {
			out.writeFileMetaInformation(fmi);
			data.copyTo(out);
		} finally {
			SafeClose.close(out);
		}
	}

	private static void renameTo(Association as, File from, File dest) throws IOException {
		if (!dest.getParentFile().mkdirs())
			dest.delete();
		if (!from.renameTo(dest))
			throw new IOException("Failed to rename " + from + " to " + dest);
	}

	private void run() throws IOException, GeneralSecurityException {

		conn.setPort(6000);
		conn.setReceivePDULength(Connection.DEF_MAX_PDU_LENGTH);
		conn.setSendPDULength(Connection.DEF_MAX_PDU_LENGTH);

		conn.setMaxOpsInvoked(0);
		conn.setMaxOpsPerformed(0);
		conn.setPackPDV(true);
		conn.setConnectTimeout(0);
		conn.setRequestTimeout(0);
		conn.setAcceptTimeout(0);
		conn.setReleaseTimeout(0);
		conn.setResponseTimeout(0);

		conn.setIdleTimeout(0);
		conn.setSocketCloseDelay(Connection.DEF_SOCKETDELAY);
		conn.setSendBufferSize(0);
		conn.setReceiveBufferSize(0);
		conn.setTcpNoDelay(true);
		this.status = 0;
		this.responseDelay = 0;

		TransferCapability tc = null;

		tc = new TransferCapability(null, UID.VerificationSOPClass, TransferCapability.Role.SCP,
				UID.ImplicitVRLittleEndian);
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StorageServiceClass, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BasicStudyContentNotificationSOPClassRetired, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StoredPrintStorageSOPClassRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.HardcopyGrayscaleImageStorageSOPClassRetired, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.HardcopyColorImageStorageSOPClassRetired, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ComputedRadiographyImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DigitalXRayImageStorageForPresentation, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DigitalXRayImageStorageForProcessing, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DigitalMammographyXRayImageStorageForPresentation,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DigitalMammographyXRayImageStorageForProcessing,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DigitalIntraOralXRayImageStorageForPresentation,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DigitalIntraOralXRayImageStorageForProcessing,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.CTImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedCTImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.LegacyConvertedEnhancedCTImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.UltrasoundMultiFrameImageStorageRetired, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.UltrasoundMultiFrameImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MRImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedMRImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MRSpectroscopyStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedMRColorImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.LegacyConvertedEnhancedMRImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.NuclearMedicineImageStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.UltrasoundImageStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.UltrasoundImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedUSVolumeStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SecondaryCaptureImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MultiFrameSingleBitSecondaryCaptureImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MultiFrameGrayscaleByteSecondaryCaptureImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MultiFrameGrayscaleWordSecondaryCaptureImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MultiFrameTrueColorSecondaryCaptureImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StandaloneOverlayStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StandaloneCurveStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.WaveformStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.TwelveLeadECGWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.GeneralECGWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.AmbulatoryECGWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.HemodynamicWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.CardiacElectrophysiologyWaveformStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BasicVoiceAudioWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.GeneralAudioWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ArterialPulseWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RespiratoryWaveformStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StandaloneModalityLUTStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StandaloneVOILUTStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.GrayscaleSoftcopyPresentationStateStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ColorSoftcopyPresentationStateStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.PseudoColorSoftcopyPresentationStateStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BlendingSoftcopyPresentationStateStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XAXRFGrayscaleSoftcopyPresentationStateStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.GrayscalePlanarMPRVolumetricPresentationStateStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.CompositingPlanarMPRVolumetricPresentationStateStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.AdvancedBlendingPresentationStateStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VolumeRenderingVolumetricPresentationStateStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SegmentedVolumeRenderingVolumetricPresentationStateStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MultipleVolumeRenderingVolumetricPresentationStateStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XRayAngiographicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedXAImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XRayRadiofluoroscopicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedXRFImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XRayAngiographicBiPlaneImageStorageRetired, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XRay3DAngiographicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XRay3DCraniofacialImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BreastTomosynthesisImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BreastProjectionXRayImageStorageForPresentation,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BreastProjectionXRayImageStorageForProcessing,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.IntravascularOpticalCoherenceTomographyImageStorageForPresentation,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.IntravascularOpticalCoherenceTomographyImageStorageForProcessing,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.NuclearMedicineImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ParametricMapStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MRImageStorageZeroPaddedRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RawDataStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SpatialRegistrationStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SpatialFiducialsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DeformableSpatialRegistrationStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SegmentationStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SurfaceSegmentationStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.TractographyResultsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RealWorldValueMappingStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SurfaceScanMeshStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SurfaceScanPointCloudStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLImageStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLMultiFrameImageStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLEndoscopicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VideoEndoscopicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLMicroscopicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VideoMicroscopicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLSlideCoordinatesMicroscopicImageStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLPhotographicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VideoPhotographicImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicPhotography8BitImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicPhotography16BitImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StereometricRelationshipStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicTomographyImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.WideFieldOphthalmicPhotographyStereographicProjectionImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.WideFieldOphthalmicPhotography3DCoordinatesImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicOpticalCoherenceTomographyEnFaceImageStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicOpticalCoherenceTomographyBScanVolumeAnalysisStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VLWholeSlideMicroscopyImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.LensometryMeasurementsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.AutorefractionMeasurementsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.KeratometryMeasurementsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SubjectiveRefractionMeasurementsStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.VisualAcuityMeasurementsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SpectaclePrescriptionReportStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicAxialMeasurementsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.IntraocularLensCalculationsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MacularGridThicknessAndVolumeReportStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicVisualFieldStaticPerimetryMeasurementsStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.OphthalmicThicknessMapStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.CornealTopographyMapStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.TextSRStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.AudioSRStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.DetailSRStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ComprehensiveSRStorageTrialRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BasicTextSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ComprehensiveSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.Comprehensive3DSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ExtensibleSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ProcedureLogStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.MammographyCADSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.KeyObjectSelectionDocumentStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ChestCADSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.XRayRadiationDoseSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RadiopharmaceuticalRadiationDoseSRStorage, TransferCapability.Role.SCP,
				"*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ColonCADSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ImplantationPlanSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.AcquisitionContextSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.SimplifiedAdultEchoSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.PatientRadiationDoseSRStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ContentAssessmentResultsStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EncapsulatedPDFStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EncapsulatedCDAStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EncapsulatedSTLStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.PositronEmissionTomographyImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.LegacyConvertedEnhancedPETImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.StandalonePETCurveStorageRetired, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.EnhancedPETImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.BasicStructuredDisplayStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.CTDefinedProcedureProtocolStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.CTPerformedProcedureProtocolStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ProtocolApprovalStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTImageStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTDoseStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTStructureSetStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTBeamsTreatmentRecordStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTPlanStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTBrachyTreatmentRecordStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTTreatmentSummaryRecordStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTIonPlanStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTIonBeamsTreatmentRecordStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTBeamsDeliveryInstructionStorageTrialRetired,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTBeamsDeliveryInstructionStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.RTBrachyApplicationSetupDeliveryInstructionStorage,
				TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.HangingProtocolStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ColorPaletteStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.GenericImplantTemplateStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ImplantAssemblyTemplateStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		tc = new TransferCapability(null, UID.ImplantTemplateGroupStorage, TransferCapability.Role.SCP, "*");
		ae.addTransferCapability(tc);
		ExecutorService executorService = Executors.newCachedThreadPool();
		ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
		device.setScheduledExecutor(scheduledExecutorService);
		device.setExecutor(executorService);
		device.bindConnections();
		closeConn = this.conn;
		running.set(true);
	}
}
